//
//  MMImageView.m
//
//  Created by Herbert Ptzl on Fri Mar 29 2002.
//  Copyright (c) 2002 H.Ptzl. 
//
//  This file is part of MathMap.
//
//  MathMap is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  MathMap is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with MathMap; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#import "MMImageView.h"

#import "MMExpression.h"
#import "MMInvocation.h"
#import "NSExtensions.h"


enum {
    ImageViewThreadInactive = 0,
    ImageViewThreadIdle = 1,
    ImageViewThreadLaunched = 2,
    ImageViewThreadRendering = 3,
    ImageViewThreadAborting = 4,
    ImageViewThreadScheduled = 5
};

@interface MMImageView (Private)

- (void)takeActions;

@end


@implementation MMImageView

- (BOOL)isFlipped 		{ return YES; }

- (void)encodeWithCoder:(NSCoder *)coder
{
    [super encodeWithCoder:coder];
//    [coder encodeConditionalObject:_target];
//    [coder encodeValueOfObjCType:@encode(SEL) at:&_action];
}

- (id)initWithCoder:(NSCoder *)coder {
    self = [super initWithCoder:coder];
//    _target = [coder decodeObject];
//    [coder decodeValueOfObjCType:@encode(SEL) at:&_action];
    return self;
}

- (MMInvocation *)invocationReuse:(BOOL)flag {
    MMInvocation *invocation = nil;
    if (_expression && _userValues) {
        if (0 && flag) {
    
        } else {
            invocation = [[MMInvocation alloc] 
                initWithExpression:_expression
                userValues:_userValues
                bitmapImageRep:_renderRep];
            [_invocation release];
        }
    } else {
/*        NSLog(@"CANT RETURN INVOCATION <E=%@, U=%@>...",
            _expression, _userValues);*/
    }
    return invocation;
}


- (void)drawRect:(NSRect)rect {
    NSRectClip(rect);

    [[NSColor checkerBoard] set];
    NSRectFill([self bounds]);

//    if (_backupIsActive) {
    if ([self inLiveResize]) {
        NSSize size = [_backupRep size];
        [_backupImage drawInRect:[self bounds] 
            fromRect:NSMakeRect(0,0, size.width, size.height) 
            operation:NSCompositeSourceOver
            fraction: 1.0];
    } else {
        NSSize size = [_renderRep size];
        [_renderImage drawInRect:[self bounds] 
            fromRect:NSMakeRect(0,0, size.width, size.height) 
            operation:NSCompositeSourceOver
            fraction: 1.0];
    }
}


- (void)enableRefreshTimer {
    if (_refreshTimer && ![_refreshTimer isValid]) {
        [_refreshTimer release];
        _refreshTimer = nil;
    }
    if (!_refreshTimer) {
        _refreshTimer = [[NSTimer 
            scheduledTimerWithTimeInterval:_refreshInterval 
            target:self selector:@selector(refresh:) 
            userInfo:nil repeats:YES] retain];
    
        [[NSRunLoop currentRunLoop] addTimer:_refreshTimer 
            forMode:NSModalPanelRunLoopMode];
        [[NSRunLoop currentRunLoop] addTimer:_refreshTimer 
            forMode:NSEventTrackingRunLoopMode];
    }
}

- (void)disableRefreshTimer {
    if (_refreshTimer) {
        if ([_refreshTimer isValid])
            [_refreshTimer invalidate];
        [_refreshTimer release];
        _refreshTimer = nil;
    }
}

- (void)updateRenderedRect {
    if ([[self window] isMiniaturized]) {
            // what else is required?
        [[self window] setMiniwindowImage:_renderImage];
    } else {
        if (_invocation) {
            NSRect rect = [_invocation renderedRect];
            NSRect newRect = NSMakeRect(
                0, NSMaxY(_updatedRect), 
                NSWidth(rect), NSMaxY(rect) - NSMaxY(_updatedRect));
            if (NSHeight(newRect)>0) {
                [self setNeedsDisplayInRect:newRect];
//                NSLog(@"update rect %@", NSStringFromRect(newRect));
                _updatedRect = NSUnionRect(newRect, _updatedRect);
            }
        } else {
//            NSLog(@"NO INVOCATION ...");
        }
    }
}

- (void)refresh:(NSTimer *)aTimer {
/*
    NSTimeInterval thisTime = [NSDate timeIntervalSinceReferenceDate];
    ... (thisTime - lastTime) ...
    lastTime = thisTime;
*/
    [self updateRenderedRect];
    [self takeActions];
}

- (BOOL)launchInvocation:(BOOL)initial {
    NSSize size = [_renderRep size];
    NSRange range = NSMakeRange(0, size.height);
    if (initial) {
        int location = MAX(0, NSMaxY(_updatedRect));
        if (location >= (int)size.height) location = 0;
        range = NSIntersectionRange(range,
            NSMakeRange(location, size.height));
        _isInitialRun = (location > 0);
/*        NSLog(@"initial(%d) launch [%d,%d]", 
            _isInitialRun, range.location, range.length);  */
    } else {
        int location = NSMinY(_updatedRect);
        range = NSMakeRange(0, location);
        _isInitialRun = NO;
/*        NSLog(@"second launch [%d,%d]", 
            range.location, range.length);		*/
    }
    _updatedRect = NSMakeRect(0, range.location, size.width, 0);
    _invocation = [self invocationReuse:NO];
    if (_invocation) {
        [_invocation setTimeValue:_timeValue];
	[_invocation setSupersampling:_flags.supersampling];
	[_invocation setAntialiasing:_flags.antialiasing];
        [_invocation renderThreaded:YES range:range];
        _state = ImageViewThreadLaunched;
        return YES;
    } else {
        _state = ImageViewThreadInactive;
        return NO;
    }
}


- (void)takeActions {
    BOOL stop = NO, release = NO, launch = NO;
    BOOL timer = NO, initial = NO;

    // NSLog(@"takeActions: state = %d", _state);

    switch (_state) {
    case ImageViewThreadInactive:
        break;

    case ImageViewThreadLaunched:
    
        _state = ImageViewThreadRendering;
        
    case ImageViewThreadRendering:
        if ([_invocation isRunning]) {
            timer = YES;
        }
        if ([_invocation hasFinished]) {
            release = YES;
        }
        break;
        
    case ImageViewThreadAborting:
        break;
        
    case ImageViewThreadScheduled:
        if (_invocation) {
            if (_syncOnFrame) {
            
            } else {
                stop = YES; release = YES;
                launch = YES; timer = YES;
            }
        } else {
            launch = YES; timer = YES;
        }
        initial = YES;
        break;
    }

    if (_invocation) {
        if (stop) [_invocation stop];
        if (release) {
            [self updateRenderedRect];
            [_invocation release];
            _invocation = nil;
            if (_isInitialRun) {
                launch = YES;
                _isInitialRun = NO;
            } else {
                _state = ImageViewThreadInactive;
                if (!launch)
                    _updatedRect = NSZeroRect;
                if (_backupIsActive) {
                    _backupIsActive = NO;
                    [self setNeedsDisplay:YES];
                    [_backupRep release];
                    [_backupImage release];
                }
            }
        }
    }
    if (launch) {
        // NSLog(@"launch ...");
        timer = [self launchInvocation:initial];
    }
    if (timer) [self enableRefreshTimer];
    else [self disableRefreshTimer];
}

- (void)viewWillStartLiveResize {
    if (!_backupIsActive) {
        _backupRep = [_renderRep retain];
        _backupImage = [_renderImage retain];
        _backupIsActive = YES;
    }
}

- (void)viewDidEndLiveResize {
}



- (void)adjustRenderImage:(NSSize)size {
    if (0 && [self inLiveResize]) {
        if (!_backupIsActive) {
            [_backupRep release];
            [_backupImage release];
            _backupRep = _renderRep;
            _backupImage = _renderImage;
            _backupIsActive = YES;
        }
    }

    _renderImage = [[NSImage alloc] initWithSize:size];
    [_renderImage setDataRetained:YES];
    [_renderImage setFlipped:YES];
    _renderRep = [[NSBitmapImageRep alloc]
        initWithBitmapDataPlanes:NULL 
        pixelsWide:size.width 
        pixelsHigh:size.height 
        bitsPerSample:8 samplesPerPixel:4 
        hasAlpha:YES isPlanar:NO 
        colorSpaceName:NSDeviceRGBColorSpace 
        bytesPerRow:0 bitsPerPixel:0];
    [_renderImage addRepresentation:_renderRep];
    [self takeActions];
}

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        _renderImage = nil;
        _renderRep = nil;

        _backupImage = nil;
        _backupRep = nil;

        _switchRect = NSZeroRect;
        _switchIsActive = NO;
        _backupIsActive = NO;
        _syncOnFrame = NO;

        _expression = nil;
        _invocation = nil;
        _userValues = nil;
    
        _timeValue = 0.0;
        _state = ImageViewThreadInactive;
        _refreshTimer = nil;
        _refreshInterval = 0.1;
        _flags.supersampling = NO;
        _flags.antialiasing = NO;
        
        [self adjustRenderImage:frame.size];
    }
    return self;
}

- (void)setFrame:(NSRect)frameRect
{
    [super setFrame:frameRect];
    [self adjustRenderImage:frameRect.size];
    [self rescheduleExpression:_expression
        userValues:_userValues];
}


-(BOOL)scheduleExpression:(MMExpression *)expression
    userValues:(NSArray *)userValues {
//    NSLog(@"SCHEDULE (%p,%p)", expression, userValues);
    [expression retain];
    [_expression release];
    _expression = expression;
    [userValues retain];
    [_userValues release];
    _userValues = userValues;
    _state = ImageViewThreadScheduled;
    [self takeActions];
    return YES;
}    
    
-(BOOL)rescheduleExpression:(MMExpression *)expression
    userValues:(NSArray *)userValues{
//    NSLog(@"RESCHEDULE (%p,%p)", expression, userValues);
    if ([_expression isEqual:expression]) {
        [userValues retain];
        [_userValues release];
        _userValues = userValues;
        _state = ImageViewThreadScheduled;
        [self takeActions];
    }
    return YES;
}    
    

-(void)stop{
    if (_invocation && [_invocation isRunning]) {
        [_invocation stop];
    }
}    
    

-(void)stopAfterFrame {
/*    if ([self isActive]) {
        _state = ImageViewThreadAborting;
    }
*/
}    
    


-(void)setSyncOnFrame:(BOOL)flag 	{ _syncOnFrame = flag; }
-(BOOL)syncsOnFrame 			{ return _syncOnFrame; }    
    
- (void)setSupersampling:(BOOL)flag	{ _flags.supersampling = flag; }
- (void)setAntialiasing:(BOOL)flag	{ _flags.antialiasing = flag; }


-(void)setRefreshInterval:(NSTimeInterval)interval {
    
}    
    

-(NSTimeInterval)refreshInterval {
    return 0.0;
}    
    


-(float)progress {
    return 0.0;
}    
    
-(NSImage *)image			{ return _renderImage; }

-(IBAction)takeFloatFrom:(id)sender {
    _timeValue = [sender floatValue];
    _state = ImageViewThreadScheduled;
    [self takeActions];
}


@end
